from keras.models import model_from_json


def save_model(model, model_path='../dnn.json'):
    # serialize model to JSON
    model_json = model.to_json()
    with open(model_path, "w") as json_file:
        json_file.write(model_json)
    print("Saved model to disk")
    return 0


def save_model_weights(model, model_weights_path='../dnn.h5'):
    # serialize weights to HDF5
    model.save_weights(model_weights_path)
    print("Saved model weights to disk")
    return 0


def load_model(model_path):
    # load json and create model
    json_file = open(model_path, 'r')
    loaded_model_json = json_file.read()
    json_file.close()
    loaded_model = model_from_json(loaded_model_json)
    return loaded_model


def load_model_weights(weights_path, model):
    # load weights into  model
    model.load_weights(weights_path)
    print("Loaded model from disk")
    return model


import h5py
from keras.utils.np_utils import to_categorical, categorical_probas_to_classes
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import LabelEncoder


def class_encoder_to_number(tobe_Y):
    encoder = LabelEncoder()
    encoder.fit(tobe_Y)
    return encoder


def class_number_encode_to_one_hot_code(class_number):
    return to_categorical(class_number)


def one_hot_code_to_class_number_encode(one_hot_code):
    return categorical_probas_to_classes(one_hot_code)


def number_to_class_name(encoder, class_number):
    return encoder.inverse_transform(class_number)


def split_dataset_to_tain_test(X, Y, test_size=0.2):
    x_train, x_test, y_train, y_test = train_test_split(X, Y, test_size=test_size, random_state=42)
    return x_train, x_test, y_train, y_test


def load_train_test_data(path='../data/pythonTutorial4/setList_mfcc.h5'):
    hfile = h5py.File(path, 'r')
    X = hfile['X'][:]
    Y = hfile['Y'][:]
    encoder = class_encoder_to_number(Y)
    Y = encoder.transform(Y)
    Y = class_number_encode_to_one_hot_code(Y)
    x_train, x_test, y_train, y_test = train_test_split(X, Y, test_size=0.2, random_state=42)

    return x_train, y_train, x_test, y_test



# !usr/bin/env python
# coding=gbk
import os
from subprocess import call

import matplotlib.pyplot as plt
import numpy as np
import pip
import scipy.io
import soundfile
from sklearn import preprocessing

INFO = "author:ZHANG Xu-long\nemail:fudan0027zxl@gmail.com\nblog:zhangxulong.site\n"

SEP = os.sep
EPSILON = 1e-8


def voteMaxTimeList(vote_list=[1, 2, 3, 3]):
    vote_list = list(vote_list)
    maxTime = max(vote_list, key=vote_list.count)
    return maxTime


def init_data_dir():
    currentPath = os.getcwd()
    projectName = currentPath.split(SEP)[-1]
    if_no_create_it('../data/' + projectName + '/')
    return '../data/' + projectName + '/'


def savefig(filename, figlist, log=True):
    h = 10
    n = len(figlist)
    # peek into instances
    f = figlist[0]
    if len(f.shape) == 1:
        plt.figure()
        for i, f in enumerate(figlist):
            plt.subplot(n, 1, i + 1)
            if len(f.shape) == 1:
                plt.plot(f)
                plt.xlim([0, len(f)])
    elif len(f.shape) == 2:
        Nsmp, dim = figlist[0].shape
        figsize = (h * float(Nsmp) / dim, len(figlist) * h)
        plt.figure(figsize=figsize)
        for i, f in enumerate(figlist):
            plt.subplot(n, 1, i + 1)
            if log:
                plt.imshow(np.log(f.T + EPSILON))
            else:
                plt.imshow(f.T + EPSILON)
    else:
        raise ValueError('Input dimension must < 3.')
    plt.savefig(filename)


def wavread(filename):
    x, fs = soundfile.read(filename)
    return x, fs


def wavwrite(filename, y, fs):
    soundfile.write(filename, y, fs)
    return 0


def print_to_check(print_list=['a', 'b']):
    for print_item in print_list:
        print(print_item)


def if_no_create_it(file_path):
    the_dir = os.path.dirname(file_path)
    if os.path.isdir(the_dir):
        pass
    else:
        os.makedirs(the_dir)


def del_the_file(file_path):
    os.remove(file_path)


def update_pip_install_packages():
    for dist in pip.get_installed_distributions():
        call("sudo pip install --upgrade " + dist.project_name, shell=True)
    return 0


def align_two_list_with_same_len(list_ref, list_tobe_modify):
    len_ref = len(list_ref)
    len_tobe_modify = len(list_tobe_modify)
    x_ref = range(len_ref)
    x_ref = [x_ref_item / (len_ref / 1.0 / len_tobe_modify) for x_ref_item in x_ref]
    x_tobe_modify = range(len_tobe_modify)
    target_y = np.interp(x_ref, x_tobe_modify, list_tobe_modify)

    return target_y


def read_mat_data(mat_file='file.mat', key='combine_feature'):
    mat = scipy.io.loadmat(mat_file)
    value = mat[key]
    # print numpy.shape(value)
    # print len(value[:,1])
    return value


def save_mat_data(mat_file='file.mat', key='combine_feature', value=''):
    mat = {key: value}
    # print mat
    # print mat_file
    if_no_create_it(mat_file)
    scipy.io.savemat(mat_file, mat)
    return 0


def normalize_data(data, method='min_max', scaler='none'):
    if method == 'min_max':
        if scaler == 'none':
            scaler = preprocessing.MinMaxScaler().fit(data)
        data = scaler.fit_transform(data)
    if method == 'mean_std':
        if scaler == 'none':
            scaler = preprocessing.StandardScaler().fit(data)
        data = scaler.fit_transform(data)

    return data, scaler


def reduceListFilter(tobeReduce=[2, 2, 1, 2, 2, 3, 3, 2, 2, 1, 8]):
    final = []
    count = 0
    tobeReduce = ['start'] + tobeReduce + ['end']
    # print tobeReduce
    for item in tobeReduce:
        if count != len(tobeReduce) - 1 and count != 0:
            if item == tobeReduce[count - 1] and item == tobeReduce[count + 1]:
                final.append(item)
        count += 1

    return final



from .baseZhang import reduceListFilter, align_two_list_with_same_len, del_the_file, if_no_create_it, init_data_dir, \
    print_to_check, \
    savefig, update_pip_install_packages, wavread, wavwrite, normalize_data, read_mat_data, save_mat_data
from .calcAudioFeature import get_audio_feature, substractFeat
from .calcChroma import calcChroma_cens, calcChroma_cqt, calcChroma_stft, calcTonnetz
from .calcLPCC import calcLPCC
from .calcMFCC import calcMFCC, calcMFCC_delta, calcMFCC_delta_delta, fbank, log_fbank, log_spectrum_power
from .calcSpec import get_spec, get_spec_2, get_spec_gammatone, plot_spec_gammatone, plot_spectrogram
from .calcSpectralFeatureLibrosa import calc_poly_features, calc_rmse, calc_spectral_bandwidth, calc_spectral_centroid, \
    calc_spectral_contrast, calc_spectral_rolloff, calc_zero_crossing_rate
from .callMatlabFunction import run_matlab_codes
from .countDays import countTwoDates
from .datasetPreprocess import class_encoder_to_number, class_number_encode_to_one_hot_code, \
    one_hot_code_to_class_number_encode, number_to_class_name, split_dataset_to_tain_test, load_train_test_data
from .fduNetwork import networkGUI
from .featureSelect import removeLowVarianceFeatures, selectKbestFeature
from .formatTrans import mp32Wav, mpeg2wav, wav2MFCC, video2mp4
from .kerasModel import load_model, load_model_weights, save_model, save_model_weights
from .labeEncodeDecode import decode, encode
from .modifyMarkdown import modify_markdown
from .plotVisualData import plot_waveform, plotDuralWav, plotmono_waveform, plotstereo_waveform, plotstft, plotMonoWav, \
    plotssd, plotmatrix
from .recordAudio import recordAudio
from .split2word import split_into_words



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Spectral feature extraction"""
import numpy as np
from librosa import util
from librosa.core.audio import zero_crossings, to_mono
from librosa.core.spectrum import power_to_db, _spectrogram
from librosa.core.time_frequency import fft_frequencies
from librosa.util.deprecation import Deprecated, rename_kw
from librosa.util.exceptions import ParameterError

__all__ = ['spectral_centroid',
           'spectral_bandwidth',
           'spectral_contrast',
           'spectral_rolloff',
           'poly_features',
           'rmse',
           'zero_crossing_rate',
           ]


# -- Spectral features -- #
def calc_spectral_centroid(y=None, sr=22050, S=None, n_fft=2048, hop_length=512,
                      freq=None):

    S, n_fft = _spectrogram(y=y, S=S, n_fft=n_fft, hop_length=hop_length)

    if not np.isrealobj(S):
        raise ParameterError('Spectral centroid is only defined '
                             'with real-valued input')
    elif np.any(S < 0):
        raise ParameterError('Spectral centroid is only defined '
                             'with non-negative energies')

    # Compute the center frequencies of each bin
    if freq is None:
        freq = fft_frequencies(sr=sr, n_fft=n_fft)

    if freq.ndim == 1:
        freq = freq.reshape((-1, 1))

    # Column-normalize S
    return np.sum(freq * util.normalize(S, norm=1, axis=0),
                  axis=0, keepdims=True)


def calc_spectral_bandwidth(y=None, sr=22050, S=None, n_fft=2048, hop_length=512,
                       freq=None, centroid=None, norm=True, p=2):

    S, n_fft = _spectrogram(y=y, S=S, n_fft=n_fft, hop_length=hop_length)

    if not np.isrealobj(S):
        raise ParameterError('Spectral bandwidth is only defined '
                             'with real-valued input')
    elif np.any(S < 0):
        raise ParameterError('Spectral bandwidth is only defined '
                             'with non-negative energies')

    if centroid is None:
        centroid = calc_spectral_centroid(y=y, sr=sr, S=S,
                                     n_fft=n_fft,
                                     hop_length=hop_length,
                                     freq=freq)

    # Compute the center frequencies of each bin
    if freq is None:
        freq = fft_frequencies(sr=sr, n_fft=n_fft)

    if freq.ndim == 1:
        deviation = np.abs(np.subtract.outer(freq, centroid[0]))
    else:
        deviation = np.abs(freq - centroid[0])

    # Column-normalize S
    if norm:
        S = util.normalize(S, norm=1, axis=0)

    return np.sum(S * deviation**p, axis=0, keepdims=True)**(1./p)


def calc_spectral_contrast(y=None, sr=22050, S=None, n_fft=2048, hop_length=512,
                      freq=None, fmin=200.0, n_bands=6, quantile=0.02,
                      linear=False):

    S, n_fft = _spectrogram(y=y, S=S, n_fft=n_fft, hop_length=hop_length)

    # Compute the center frequencies of each bin
    if freq is None:
        freq = fft_frequencies(sr=sr, n_fft=n_fft)

    freq = np.atleast_1d(freq)

    if freq.ndim != 1 or len(freq) != S.shape[0]:
        raise ParameterError('freq.shape mismatch: expected '
                             '({:d},)'.format(S.shape[0]))

    if n_bands < 1 or not isinstance(n_bands, int):
        raise ParameterError('n_bands must be a positive integer')

    if not 0.0 < quantile < 1.0:
        raise ParameterError('quantile must lie in the range (0, 1)')

    if fmin <= 0:
        raise ParameterError('fmin must be a positive number')

    octa = np.zeros(n_bands + 2)
    octa[1:] = fmin * (2.0**np.arange(0, n_bands + 1))

    if np.any(octa[:-1] >= 0.5 * sr):
        raise ParameterError('Frequency band exceeds Nyquist. '
                             'Reduce either fmin or n_bands.')

    valley = np.zeros((n_bands + 1, S.shape[1]))
    peak = np.zeros_like(valley)

    for k, (f_low, f_high) in enumerate(zip(octa[:-1], octa[1:])):
        current_band = np.logical_and(freq >= f_low, freq <= f_high)

        idx = np.flatnonzero(current_band)

        if k > 0:
            current_band[idx[0] - 1] = True

        if k == n_bands:
            current_band[idx[-1] + 1:] = True

        sub_band = S[current_band]

        if k < n_bands:
            sub_band = sub_band[:-1]

        # Always take at least one bin from each side
        idx = np.rint(quantile * np.sum(current_band))
        idx = int(np.maximum(idx, 1))

        sortedr = np.sort(sub_band, axis=0)

        valley[k] = np.mean(sortedr[:idx], axis=0)
        peak[k] = np.mean(sortedr[-idx:], axis=0)

    if linear:
        return peak - valley
    else:
        return power_to_db(peak) - power_to_db(valley)


def calc_spectral_rolloff(y=None, sr=22050, S=None, n_fft=2048, hop_length=512,
                     freq=None, roll_percent=0.85):

    if not 0.0 < roll_percent < 1.0:
        raise ParameterError('roll_percent must lie in the range (0, 1)')

    S, n_fft = _spectrogram(y=y, S=S, n_fft=n_fft, hop_length=hop_length)

    if not np.isrealobj(S):
        raise ParameterError('Spectral rolloff is only defined '
                             'with real-valued input')
    elif np.any(S < 0):
        raise ParameterError('Spectral rolloff is only defined '
                             'with non-negative energies')

    # Compute the center frequencies of each bin
    if freq is None:
        freq = fft_frequencies(sr=sr, n_fft=n_fft)

    # Make sure that frequency can be broadcast
    if freq.ndim == 1:
        freq = freq.reshape((-1, 1))

    total_energy = np.cumsum(S, axis=0)

    threshold = roll_percent * total_energy[-1]

    ind = np.where(total_energy < threshold, np.nan, 1)

    return np.nanmin(ind * freq, axis=0, keepdims=True)


def calc_rmse(y=None, S=None, frame_length=2048, hop_length=512,
         center=True, pad_mode='reflect',
         n_fft=Deprecated()):

    frame_length = rename_kw('n_fft', n_fft,
                             'frame_length', frame_length,
                             '0.5', '0.6')

    if y is not None and S is not None:
        raise ValueError('Either `y` or `S` should be input.')
    if y is not None:
        y = to_mono(y)
        if center:
            y = np.pad(y, int(frame_length // 2), mode=pad_mode)

        x = util.frame(y,
                       frame_length=frame_length,
                       hop_length=hop_length)
    elif S is not None:
        x, _ = _spectrogram(y=y, S=S,
                            n_fft=frame_length,
                            hop_length=hop_length)
    else:
        raise ValueError('Either `y` or `S` must be input.')
    return np.sqrt(np.mean(np.abs(x)**2, axis=0, keepdims=True))


def calc_poly_features(y=None, sr=22050, S=None, n_fft=2048, hop_length=512,
                  order=1, freq=None):

    S, n_fft = _spectrogram(y=y, S=S, n_fft=n_fft, hop_length=hop_length)

    # Compute the center frequencies of each bin
    if freq is None:
        freq = fft_frequencies(sr=sr, n_fft=n_fft)

    # If frequencies are constant over frames, then we only need to fit once
    if freq.ndim == 1:
        coefficients = np.polyfit(freq, S, order)
    else:
        # Else, fit each frame independently and stack the results
        coefficients = np.concatenate([[np.polyfit(freq[:, i], S[:, i], order)]
                                       for i in range(S.shape[1])], axis=0).T

    return coefficients


def calc_zero_crossing_rate(y, frame_length=2048, hop_length=512, center=True,
                       **kwargs):

    util.valid_audio(y)

    if center:
        y = np.pad(y, int(frame_length // 2), mode='edge')

    y_framed = util.frame(y, frame_length, hop_length)

    kwargs['axis'] = 0
    kwargs.setdefault('pad', False)

    crossings = zero_crossings(y_framed, **kwargs)

    return np.mean(crossings, axis=0, keepdims=True)






import numpy as np
import pymir
from scipy.linalg import toeplitz
import itertools

def lpc(seq, order=None):
    '''
    Function: lpc
    Summary: Computes for each given sequence the LPC ( Linear predictive components ) sequence.
    Examples:
    Attributes:
        @param (seq):A sequence of time-domain frames, usually obtained by .frames()
        @param (order) default=None: Size of the returning cepstral components. If None is given,
                                     we use len(seq) as default, otherwise order +1
    Returns: A tuple, which elements are (lpc coefficents,error_term). The error term is the sqare root of the squared prediction error.
    '''
    # In this lpc method we use the slow( if the order is >50) autocorrelation approach.
    acseq = np.array(pymir.Frame.autocorr(seq, order))
    # Using pseudoinverse to obtain a stable estiamte of the toeplitz matrix
    a_coef = np.dot(np.linalg.pinv(toeplitz(acseq[:-1])), -acseq[1:].T)
    # Squared prediction error, defined as e[n] = a[n] + \sum_k=1^order (a_k *
    # s_{n-k})
    err_term = acseq[0] + sum(a * c for a, c in zip(acseq[1:], a_coef))
    return a_coef.tolist(), np.sqrt(err_term)


def lpcc(seq, err_term, order=None):
    '''
    Function: lpcc
    Summary: Computes the linear predictive cepstral compoents. Note: Returned values are in the frequency domain
    Examples: audiofile = AudioFile.open('file.wav',16000)
              frames = audiofile.frames(512,np.hamming)
              for frame in frames:
                frame.lpcc()
              Note that we already preprocess in the Frame class the lpc conversion!
    Attributes:
        @param (seq):A sequence of lpc components. Need to be preprocessed by lpc()
        @param (err_term):Error term for lpc sequence. Returned by lpc()[1]
        @param (order) default=None: Return size of the array. Function returns order+1 length array. Default is len(seq)
    Returns: List with lpcc components with default length len(seq), otherwise length order +1
    '''
    if order is None:
        order = len(seq) - 1
    lpcc_coeffs = [np.log(err_term), -seq[0]]
    for n in xrange(2, order + 1):
        # Use order + 1 as upper bound for the last iteration
        upbound = (order + 1 if n > order else n)
        lpcc_coef = -sum(i * lpcc_coeffs[i] * seq[n - i - 1]
                         for i in xrange(1, upbound)) * 1. / upbound
        lpcc_coef -= seq[n - 1] if n <= len(seq) else 0
        lpcc_coeffs.append(lpcc_coef)
    return lpcc_coeffs

def lsp(lpcseq,rectify=True):
    '''
    Function: lsp
    Summary: Computes Line spectrum pairs ( also called  line spectral frequencies [lsf]). Does not use any fancy algorithm except np.roots to solve
    for the zeros of the given polynom A(z) = 0.5(P(z) + Q(z))
    Examples: audiofile = AudioFile.open('file.wav',16000)
              frames = audiofile.frames(512,np.hamming)
              for frame in frames:
                frame.lpcc()
    Attributes:
        @param (lpcseq):The sequence of lpc coefficients as \sum_k=1^{p} a_k z^{-k}
        @param (rectify) default=True: If true returns only the values >= 0, since the result is symmetric. If all values are wished, specify rectify = False
    Returns: A list with same length as lpcseq (if rectify = True), otherwise 2*len(lpcseq), which represents the line spectrum pairs
    '''
    # We obtain 1 - A(z) +/- z^-(p+1) * (1 - A(z))
    # After multiplying everything through it becomes
    # 1 - \sum_k=1^p a_k z^{-k} +/- z^-(p+1) - \sum_k=1^p a_k z^{k-(p+1)}
    # Thus we have on the LHS the usual lpc polynomial and on the RHS we need to reverse the coefficient order
    # We assume further that the lpc polynomial is a valid one ( first coefficient is 1! )

    # the rhs does not have a constant expression and we reverse the coefficients
    rhs = [0] + lpcseq[::-1] + [1]
    # The P polynomial
    P = []
    # The Q polynomial
    Q = []
    # Assuming constant coefficient is 1, which is required. Moreover z^{-p+1} does not exist on the lhs, thus appending 0
    lpcseq = [1] + lpcseq[:] + [0]
    for l,r in itertools.izip_longest(lpcseq,rhs):
        P.append(l + r)
        Q.append(l - r)
    # Find the roots of the polynomials P,Q ( numpy assumes we have the form of: p[0] * x**n + p[1] * x**(n-1) + ... + p[n-1]*x + p[n]
    # mso we need to reverse the order)
    p_roots = np.roots(P[::-1])
    q_roots = np.roots(Q[::-1])
    # Keep the roots in order
    lsf_p = sorted(np.angle(p_roots))
    lsf_q = sorted(np.angle(q_roots))
    # print sorted(lsf_p+lsf_q),len([i for  i in lsf_p+lsf_q if i > 0.])
    if rectify:
        # We only return the positive elements, and also remove the final Pi (3.14) value at the end,
        # since it always occurs
        return sorted(i for i in lsf_q + lsf_p if (i > 0))[:-1]
    else:
        # Remove the -Pi and +pi at the beginning and end in the list
        return sorted(i for i in lsf_q + lsf_p)[1:-1]



from sklearn.feature_selection import SelectKBest
from sklearn.feature_selection import VarianceThreshold
from sklearn.feature_selection import chi2


def removeLowVarianceFeatures(feature=[[1, 0, 1], [0, 1, 0]]):
    sel = VarianceThreshold(threshold=(.8 * (1 - .8)))
    feature = sel.fit_transform(feature)
    return feature


def selectKbestFeature(featureX, targetY, KbestNum):
    feature = SelectKBest(chi2, KbestNum).fit_transform(featureX, targetY)
    return feature





import gammatone.gtgram as gmt
import pylab
import scipy.io.wavfile as wav
import stft
from gammatone.plot import render_audio_from_file
from scipy.io import wavfile


def get_wav_info(wav_file):
    rate, data = wavfile.read(wav_file)
    return rate, data


def get_spec(wavPath):
    frame_rate, sound_info = get_wav_info(wavPath)
    # print frame_rate
    Pxx, freqs, bins, im = pylab.specgram(sound_info, Fs=frame_rate)
    return Pxx


def get_spec_2(wav_path):
    fs, audio = wav.read(wav_path)
    specgram = stft.spectrogram(audio, 256)

    return specgram


def get_spec_gammatone(wav_path):
    fs, audio = wav.read(wav_path)
    win_size = 0.08  # s
    hop_len = 0.04  # s
    channels = 1024
    f_min = 20
    specgram = gmt.gtgram(audio, fs, win_size, hop_len, channels, f_min)
    return specgram


def plot_spec_gammatone(wav_path):
    render_audio_from_file(wav_path, None, gmt.gtgram)
    return 0

def plot_spectrogram(wav_file):
    sound_info, frame_rate = get_wav_info(wav_file)
    pylab.figure(num=None, figsize=(19, 12))
    pylab.subplot(111)
    pylab.title('spectrogram of %r' % wav_file)
    pylab.specgram(sound_info, Fs=frame_rate)
    pylab.savefig('spectrogram.png')



# coding=utf-8
import time
from Tkinter import *

import requests


def networkGUI():
    def login():
        username = link_contend.get()
        password = pass_contend.get()
        data = {'username': username, 'password': '{TEXT}' + password, 'n': 100, 'type': 1, 'drop': 0}
        baidu = requests.get("http://baidu.com")
        # print baidu.text
        if 'http://10.108.255.249/' in baidu.text:
            print "yes no network"
            r = requests.post("http://10.108.255.249/cgi-bin/do_login", data=data)
            print(r.text)
            print 'ok'
        return 0

    def dead_loop():

        login()
        while True:
            secs = time.time()
            now = time.strftime("%M", time.localtime(secs))
            print time.strftime("%y-%m-%d::%H:%M", time.localtime(secs))

            if 30 < int(now) < 40:
                print 'try to login'
                login()
                time.sleep(600)
            else:
                time.sleep(600)

    root = Tk()
    root.title('复旦校园网自动认证检测小助手')

    labe_txt = Label(root, text='用户名：')
    labe_txt.grid(row=0, column=0)
    entry_link = Entry(root, width=20)
    entry_link.grid(row=0, column=1, columnspan=2)
    link_contend = StringVar()
    entry_link.config(textvariable=link_contend)
    link_contend.set('12210240076')

    labe_txt = Label(root, text='密 码：')
    labe_txt.grid(row=1, column=0)
    entry_link = Entry(root, width=20)
    entry_link.grid(row=1, column=1, columnspan=2)
    pass_contend = StringVar()
    entry_link.config(textvariable=pass_contend)
    pass_contend.set('130629')

    start_button = Button(root, text='网络监测', command=dead_loop, width=30)
    start_button.grid(row=2, column=0, columnspan=3)

    info_entry = Text(root, width=20, height=3, )
    info_entry.grid(row=3, column=0, columnspan=3, rowspan=3)
    info_entry.tag_config('a', foreground='green')
    info_entry.config(font='helvetica 18')
    info_entry.insert(1.0,
                      '输入用户名和密码后点击网络检测，\n就会开始死循环，\n当网络掉线就会尝试自动登录By © mirlab\n',
                      'a')

    mainloop()
    return 0


if __name__ == '__main__':
    networkGUI()



"""
Transforms for converting between time and spectral domains
Includes: FFT/IFFT, DCT/IDCT, CQT
Last updated: 15 December 2012
"""

import numpy
import numpy.fft
from numpy import *

import scipy
from scipy.fftpack import *

import pymir

# Fourier Transforms
def fft(frame):
	"""
	Compute the spectrum using an FFT
	Returns an instance of Spectrum
	"""
	fftdata = numpy.fft.rfft(frame) # rfft only returns the real half of the FFT values, which is all we need
	spectrum = fftdata.view(pymir.Spectrum)
	spectrum.sampleRate = frame.sampleRate
	return spectrum

def ifft(spectrum):
	"""
	Compute the Inverse FFT
	"""
	fftdata = numpy.fft.irfft(spectrum)
	frame = fftdata.view(pymir.Frame)
	frame.sampleRate = spectrum.sampleRate
	return frame

# Discrete Cosine Transforms (DCT)
def dct(frame):
	"""
	Compute the Discrete Cosine Transform (DCT)
	"""
	dctResult = scipy.fftpack.dct(frame, type = 2, norm = 'ortho')
	dctSpectrum = dctResult.view(pymir.Spectrum)
	dctSpectrum.sampleRate = frame.sampleRate
	return dctSpectrum

def idct(spectrum):
	"""
	Compute the Inverse Discrete Cosine Transform (IDCT)
 	"""
	idctResult = scipy.fftpack.idct(spectrum, type = 2, norm = 'ortho')
	idctFrame = idctResult.view(pymir.Frame)
	idctFrame.sampleRate = spectrum.sampleRate
	return idctFrame

# Constant Q Transform
def cqt(frame):
	"""
	Compute the Constant Q Transform (CQT)
	"""
	N = len(frame)
	y = array(zeros(N))
	a = sqrt(2 / float(N))
	for k in range(N):
 		for n in range(N):
			y[k] += frame[n] * cos(pi * (2 * n + 1) * k / float(2 * N))
			
			if k == 0:
				y[k] = y[k] * sqrt(1 / float(N))
			else:
				y[k] = y[k] * a
	
	return y


import math
import os

import h5py
import numpy

import fromLibrosa
from baseZhang import wavread
from baseZhang import wavwrite, print_to_check
from calcMFCC import calcMFCC, calcMFCC_delta, calcMFCC_delta_delta
from calcSpectralFeatureLibrosa import calc_spectral_bandwidth, calc_spectral_contrast, calc_poly_features, \
    calc_spectral_rolloff, calc_spectral_centroid, calc_rmse, calc_zero_crossing_rate
from pymir import AudioFile


def get_dataset_X_Y(dataset='dataset.h5', dataX='X', dataY='Y'):
    h5file = h5py.File(dataset, 'r')
    X = h5file[dataX][:]
    Y = h5file[dataY][:]
    h5file.close()

    return X, Y


def substractFeat(trainX, trainY, subFeatList=['mfcc', 'chroma', 'lpcc']):
    subFeatTrainX = []
    subFeatTrainY = []
    # print numpy.shape(trainX)
    # print numpy.shape(trainY)
    # print type(trainX), type(trainY)
    for itemX, itemY in zip(trainX, trainY):
        featDict = {'mfcc': itemX[:26],
                    'chroma': itemX[26:38],
                    'zcr': itemX[38:39],
                    'rmse': itemX[39:40],
                    'centroid': itemX[40:41],
                    'rolloff': itemX[41:42],
                    'poly': itemX[42:44],
                    'contrast': itemX[44:51],
                    'bandwith': itemX[51:52],
                    'mfcc_delta_delta': itemX[52:91],
                    'mfcc_delta': itemX[91:117],
                    'lpcc': itemX[117:]}
        combine_item = featDict[subFeatList[0]]
        for item_feat in subFeatList[1:]:
            combine_item = numpy.hstack((combine_item, featDict[item_feat]))
        drop_flag = False
        for number in combine_item:
            if math.isinf(number) or math.isnan(number):
                drop_flag = True
                break
        if not drop_flag:
            subFeatTrainX.append(combine_item)
            subFeatTrainY.append(itemY)

    return numpy.array(subFeatTrainX), numpy.array(subFeatTrainY)


def substractFeatTimeStep(trainX, trainY, subFeatList=['mfcc', 'chroma', 'lpcc']):
    subFeatTrainX = []
    subFeatTrainY = []
    for trainX_step, trainY_step in zip(trainX, trainY):
        subFeatTrainX_step = []
        subFeatTrainY_step = []
        drop_flag = False
        for itemX in trainX_step:
            featDict = {'mfcc': itemX[:26],
                        'chroma': itemX[26:38],
                        'zcr': itemX[38:39],
                        'rmse': itemX[39:40],
                        'centroid': itemX[40:41],
                        'rolloff': itemX[41:42],
                        'poly': itemX[42:44],
                        'contrast': itemX[44:51],
                        'bandwith': itemX[51:52],
                        'mfcc_delta_delta': itemX[52:91],
                        'mfcc_delta': itemX[91:117],
                        'lpcc': itemX[117:]}
            combine_item = featDict[subFeatList[0]]
            for item_feat in subFeatList[1:]:
                combine_item = numpy.hstack((combine_item, featDict[item_feat]))

            for number in combine_item:
                if math.isinf(number) or math.isnan(number):
                    drop_flag = True

            subFeatTrainX_step.append(list(combine_item))
            subFeatTrainY_step.append(trainY_step)
        subFeatTrainX_step = numpy.array(subFeatTrainX_step)
        subFeatTrainY_step = numpy.array(subFeatTrainY_step)
        if not drop_flag:
            subFeatTrainX.append(subFeatTrainX_step)
            subFeatTrainY.append(subFeatTrainY_step[0])
    return numpy.array(subFeatTrainX), numpy.array(subFeatTrainY)


def test_substractFeat():
    '''['mfcc', 'lpcc', 'chroma', 'zcr', 'rmse', 'centroid', 'rolloff', 'poly', 'contrast', 'bandwith',
                   'mfcc_delta_delta', 'mfcc_delta']'''
    useDataset = '../data/sing_voice_detection/test_dataset_all_feat.h5'
    X, Y = get_dataset_X_Y(useDataset)
    subFeatList = ['mfcc', 'lpcc', 'chroma', 'zcr', 'rmse', 'centroid', 'rolloff', 'poly', 'contrast', 'bandwith'
                   ]
    subFeatTrainX, subFeatTrainY = substractFeat(X, Y, subFeatList)
    print numpy.shape(subFeatTrainY), numpy.shape(subFeatTrainX)
    print type(Y), type(subFeatTrainY)
    return 0


def get_audio_feature(test_wav='../test.wav', WIN_SIZE=0.025, HOP_LEN=0.025,
                      feat_list=['mfcc', 'lpcc', 'chroma', 'zcr', 'rmse', 'centroid', 'rolloff', 'poly', 'contrast',
                                 'bandwith', 'mfcc_delta_delta', 'mfcc_delta']):
    # WIN_SIZE need equls to HOP_LEN ie. no overlaps between frames.
    FEAT_shapes = []
    all_feat = []
    '''
    mfcc,chroma,zcr,rmse,centroid,rolloff,poly,contrast,bandwith,mfcc_delta_delta,mfcc_delta,lpcc

    '''

    audio_data, sample_rate = wavread(test_wav)
    # print sample_rate
    print 'mfcc'
    featMFCC = calcMFCC(audio_data, sample_rate, appendEnergy=False, cep_num=26, win_length=WIN_SIZE, win_step=HOP_LEN)
    FEAT_shapes.append(numpy.shape(featMFCC))
    if 'mfcc' in feat_list:
        all_feat.append(featMFCC)
    print 'chroma'
    featChroma = fromLibrosa.chroma_stft(y=audio_data, sr=sample_rate, hop_length=int(HOP_LEN * sample_rate),
                                         n_fft=int(WIN_SIZE * sample_rate)).T
    FEAT_shapes.append(numpy.shape(featChroma))
    if 'chroma' in feat_list:
        all_feat.append(featChroma)
    print 'zcr'
    featZCR = calc_zero_crossing_rate(audio_data, hop_length=int(HOP_LEN * sample_rate)).T
    FEAT_shapes.append(numpy.shape(featZCR))
    if 'zcr' in feat_list:
        all_feat.append(featZCR)
    print 'rmse'
    featRMSE = calc_rmse(audio_data, frame_length=int(WIN_SIZE * sample_rate), hop_length=int(HOP_LEN * sample_rate)).T[
               :-1]
    FEAT_shapes.append(numpy.shape(featRMSE))
    if 'rmse' in feat_list:
        all_feat.append(featRMSE)
    print 'centroid'
    featCentroid = calc_spectral_centroid(audio_data, sample_rate, hop_length=int(HOP_LEN * sample_rate),
                                          n_fft=int(WIN_SIZE * sample_rate)).T
    FEAT_shapes.append(numpy.shape(featCentroid))
    if 'centroid' in feat_list:
        all_feat.append(featCentroid)
    print 'rolloff'
    featRolloff = calc_spectral_rolloff(audio_data, sample_rate, hop_length=int(HOP_LEN * sample_rate),
                                        n_fft=int(WIN_SIZE * sample_rate)).T
    FEAT_shapes.append(numpy.shape(featRolloff))
    if 'rolloff' in feat_list:
        all_feat.append(featRolloff)
    print 'poly'
    featPoly = calc_poly_features(audio_data, sample_rate, hop_length=int(HOP_LEN * sample_rate),
                                  n_fft=int(WIN_SIZE * sample_rate)).T
    FEAT_shapes.append(numpy.shape(featPoly))
    if 'poly' in feat_list:
        all_feat.append(featPoly)
    print 'contrast'
    featContrast = calc_spectral_contrast(audio_data, sample_rate, hop_length=int(HOP_LEN * sample_rate),
                                          n_fft=int(WIN_SIZE * sample_rate)).T
    FEAT_shapes.append(numpy.shape(featContrast))
    if 'contrast' in feat_list:
        all_feat.append(featContrast)
    print 'bandwith'
    featBandwith = calc_spectral_bandwidth(audio_data, sample_rate, hop_length=int(HOP_LEN * sample_rate),
                                           n_fft=int(WIN_SIZE * sample_rate)).T
    FEAT_shapes.append(numpy.shape(featBandwith))
    if 'bandwith' in feat_list:
        all_feat.append(featBandwith)
    print 'mfcc_delta_delta'
    featMFCC_delta_delta = calcMFCC_delta_delta(audio_data, sample_rate, appendEnergy=False, cep_num=13,
                                                win_length=WIN_SIZE, win_step=HOP_LEN)
    FEAT_shapes.append(numpy.shape(featMFCC_delta_delta))
    if 'mfcc_delta_delta' in feat_list:
        all_feat.append(featMFCC_delta_delta)
    print 'mfcc_delta'
    featMFCC_delta = calcMFCC_delta(audio_data, sample_rate, appendEnergy=False, cep_num=13,
                                    win_length=WIN_SIZE, win_step=HOP_LEN)
    FEAT_shapes.append(numpy.shape(featMFCC_delta))
    if 'mfcc_delta' in feat_list:
        all_feat.append(featMFCC_delta)
    print 'lpcc'
    ## calc lpcc
    temp_wav = 'temp.wav'
    wavwrite(temp_wav, audio_data, sample_rate)
    audio_data = AudioFile.open(temp_wav)
    os.remove(temp_wav)
    lpcc_order = 12
    featLPCC = []
    # print audio_data[1]
    windowFunction = numpy.hamming
    fixedFrames = audio_data.frames(int(WIN_SIZE * sample_rate), windowFunction)
    for frame_item in fixedFrames:
        frame_lpcc_feature = frame_item.lpcc(lpcc_order)
        featLPCC.append(frame_lpcc_feature)
    featLPCC = numpy.array(featLPCC)
    ## calc lpcc end

    FEAT_shapes.append(numpy.shape(featLPCC))
    if 'lpcc' in feat_list:
        all_feat.append(featLPCC)
    min_shapes = FEAT_shapes[0][0]
    for shape_item in FEAT_shapes[1:]:
        if shape_item[0] < min_shapes:
            min_shapes = shape_item[0]

    print_to_check(FEAT_shapes)
    combineFeat = all_feat[0][:min_shapes]
    for feat_item in all_feat[1:]:
        combineFeat = numpy.hstack((combineFeat[:min_shapes], feat_item[:min_shapes]))
    print 'combine:', numpy.shape(combineFeat)
    # drop_flag = False
    # for number in frame_lpcc_feature:
    #     if math.isinf(number) or math.isnan(number):
    #         drop_flag = True
    #         break
    # if not drop_flag:
    #     featLPCC.append(frame_lpcc_feature)
    return combineFeat


if __name__ == '__main__':
    combineFeat = get_audio_feature()



# !usr/bin/env python
# coding=gbk
import os


def modify_markdown(path_to_process='post'):
    for parent, dirnames, filenames in os.walk(path_to_process):
        for filename in filenames:
            file_path = os.path.join(parent, filename)
            file_md = open(file_path, 'r')
            all_lines = file_md.readlines()
            file_md.close()
            for line in all_lines:
                if 'category: ' in line:
                    value = line
                    insert_index = all_lines.index(value)
                    all_lines.remove(value)
                    all_lines.insert(insert_index, '- ' + value.split(' ')[1] + '\n')
                    all_lines.insert(insert_index, 'categories:\n')
            file_md = open(file_path, 'w')
            file_md.writelines(all_lines)
    return 0



import numpy as np


def getDeltas(seq, derivative=2, winsize=2):
    # First stack the static features
    ret = seq[:]
    for i in xrange(derivative):
        seq = _getSingleDeltas(seq)
        ret.extend(seq)
    return ret


def _getSingleDeltas(feature, winsize=2):
    '''
    Calculates a single pass deltas for the given feature
    returns the calculated feature stacked upon the given feature
    '''
    ret = []
    # Calculates the denominator: 2* \sum_n^N n*n
    denom = 2. * sum(x**2 for x in xrange(1, winsize + 1))
    # iterate over all frames
    for frameindex in xrange(len(feature)):
        # We calculate the difference in between two frames
        # In the border case of having the current frame is < winsize, we use the
        # Current frame as the "replacement" effectively exting the array left and right by
        # the frames at the positions +- winsize
        fwd = bwd = feature[frameindex]
        innersum = 0
        # Winsize will range between 1 and winsize+1, since we want to have the
        # adjacent frames
        for k in xrange(1, winsize + 1):
            # Check if our features are in range, if not we use the default
            # setting
            # Since one of the features will certainly be not out of range (
            # except having
            # a zero or one length frame length), we don't get any zeros in the
            # result
            if frameindex + k < len(feature):
                fwd = feature[frameindex + k]
            if frameindex - k >= 0:
                bwd = feature[frameindex - k]
            innersum += k * (fwd - bwd)
        ret.append(innersum / denom)
    return ret



# !usr/bin/env python
# coding=gbk
import pylab as pl
import matplotlib.pyplot as plt
import pylab
import numpy as np
from numpy.lib import stride_tricks
import wave
# PLOTTING FUNCTIONS for RP_EXTRACT features and Audio Waveforms
def plotmatrix(features, xlabel=None, ylabel=None):
    pylab.figure()
    pylab.imshow(features, origin='lower', aspect='auto', interpolation='nearest')
    if xlabel: plt.xlabel(xlabel)
    if ylabel: pylab.ylabel(ylabel)
    pylab.show()


def plotrp(features, reshape=True, rows=24, cols=60):
    if reshape:
        features = features.reshape(rows, cols, order='F')
    plotmatrix(features, 'Modulation Frequency Index', 'Frequency Band [Bark]')


def plotssd(features, reshape=True, rows=24, cols=7):
    if reshape:
        features = features.reshape(rows, cols, order='F')

    pylab.figure()
    pylab.imshow(features, origin='lower', aspect='auto', interpolation='nearest')
    pylab.xticks(range(0, cols), ['mean', 'var', 'skew', 'kurt', 'median', 'min', 'max'])
    pylab.ylabel('Frequency [Bark]')
    pylab.show()


def plotrh(hist, showbpm=True):
    xrange = range(0, hist.shape[0])
    plt.bar(xrange, hist)  # 50, normed=1, facecolor='g', alpha=0.75)

    # plt.ylabel('Probability')
    plt.title('Rhythm Histogram')
    if showbpm:
        mod_freq_res = 1.0 / (2 ** 18 / 44100.0)
        # print type(xrange)
        plotrange = range(1, hist.shape[0] + 1, 5)  # 5 = step
        bpm = np.around(np.array(plotrange) * mod_freq_res * 60.0, 0)
        pylab.xticks(plotrange, bpm)
        plt.xlabel('bpm')
    else:
        plt.xlabel('Mod. Frequency Index')
    plt.show()


def plotmono_waveform(samples, plot_width=6, plot_height=4):
    fig = plt.figure(num=None, figsize=(plot_width, plot_height), dpi=72, facecolor='w', edgecolor='k')

    if len(samples.shape) > 1:
        # if we have more than 1 channel, build the average
        samples_to_plot = samples.copy().mean(axis=1)
    else:
        samples_to_plot = samples

    channel_1 = fig.add_subplot(111)
    channel_1.set_ylabel('Channel 1')
    # channel_1.set_xlim(0,song_length) # todo
    channel_1.set_ylim(-1, 1)

    channel_1.plot(samples_to_plot)

    plt.show();
    plt.clf();


def plotstereo_waveform(samples, plot_width=6, plot_height=5):
    fig = plt.figure(num=None, figsize=(plot_width, plot_height), dpi=72, facecolor='w', edgecolor='k')

    channel_1 = fig.add_subplot(211)
    channel_1.set_ylabel('Channel 1')
    # channel_1.set_xlim(0,song_length) # todo
    channel_1.set_ylim(-1, 1)
    channel_1.plot(samples[:, 0])

    channel_2 = fig.add_subplot(212)
    channel_2.set_ylabel('Channel 2')
    channel_2.set_xlabel('Time (s)')
    channel_2.set_ylim(-1, 1)
    # channel_2.set_xlim(0,song_length) # todo
    channel_2.plot(samples[:, 1])

    plt.show();
    plt.clf();


def plot_waveform(samples, plot_width=6, plot_height=4):
    # mono wave data is either only 1dim in shape or has a 2dim shape with 1 channel only
    if (len(samples.shape) == 1) or (samples.shape[1] == 1):
        print "Plotting Mono"
        plotmono_waveform(samples, plot_width, plot_height)
    else:
        print "Plotting Stereo"
        plotstereo_waveform(samples, plot_width, plot_height)


""" scale frequency axis logarithmically """


def logscale_spec(spec, sr=44100, factor=20.):
    timebins, freqbins = np.shape(spec)

    scale = np.linspace(0, 1, freqbins) ** factor
    scale *= (freqbins - 1) / max(scale)
    scale = np.unique(np.round(scale))

    # create spectrogram with new freq bins
    newspec = np.complex128(np.zeros([timebins, len(scale)]))
    for i in range(0, len(scale)):
        if i == len(scale) - 1:
            newspec[:, i] = np.sum(spec[:, scale[i]:], axis=1)
        else:
            newspec[:, i] = np.sum(spec[:, scale[i]:scale[i + 1]], axis=1)

    # list center freq of bins
    allfreqs = np.abs(np.fft.fftfreq(freqbins * 2, 1. / sr)[:freqbins + 1])
    freqs = []
    for i in range(0, len(scale)):
        if i == len(scale) - 1:
            freqs += [np.mean(allfreqs[scale[i]:])]
        else:
            freqs += [np.mean(allfreqs[scale[i]:scale[i + 1]])]

    return newspec, freqs


def stft(sig, frameSize, overlapFac=0.5, window=np.hanning):
    win = window(frameSize)
    hopSize = int(frameSize - np.floor(overlapFac * frameSize))

    # zeros at beginning (thus center of 1st window should be for sample nr. 0)
    samples = np.append(np.zeros(np.floor(frameSize / 2.0)), sig)
    # cols for windowing
    cols = np.ceil((len(samples) - frameSize) / float(hopSize)) + 1
    # zeros at end (thus samples can be fully covered by frames)
    samples = np.append(samples, np.zeros(frameSize))

    frames = stride_tricks.as_strided(samples, shape=(cols, frameSize),
                                      strides=(samples.strides[0] * hopSize, samples.strides[0])).copy()
    frames *= win

    return np.fft.rfft(frames)


def plotstft(samples, samplerate, binsize=2 ** 10, plotpath=None, colormap="jet", ax=None, fig=None, plot_width=6,
             plot_height=4, ignore=False):
    if ignore:
        import warnings

        warnings.filterwarnings('ignore')

    s = stft(samples, binsize)

    sshow, freq = logscale_spec(s, factor=1.0, sr=samplerate)
    ims = 20. * np.log10(np.abs(sshow) / 10e-6)  # amplitude to decibel

    timebins, freqbins = np.shape(ims)

    if ax is None:
        fig, ax = plt.subplots(1, 1, sharey=True, figsize=(plot_width, plot_height))

    # ax.figure(figsize=(15, 7.5))
    cax = ax.imshow(np.transpose(ims), origin="lower", aspect="auto", cmap=colormap, interpolation="none")
    # cbar = fig.colorbar(cax, ticks=[-1, 0, 1], cax=ax)
    # ax.set_colorbar()

    ax.set_xlabel("time (s)")
    ax.set_ylabel("frequency (hz)")
    ax.set_xlim([0, timebins - 1])
    ax.set_ylim([0, freqbins])

    xlocs = np.float32(np.linspace(0, timebins - 1, 5))
    ax.set_xticks(xlocs, ["%.02f" % l for l in ((xlocs * len(samples) / timebins) + (0.5 * binsize)) / samplerate])
    ylocs = np.int16(np.round(np.linspace(0, freqbins - 1, 10)))
    ax.set_yticks(ylocs, ["%.02f" % freq[i] for i in ylocs])

    if plotpath:
        plt.savefig(plotpath, bbox_inches="tight")
    else:
        plt.show()

    # plt.clf();
    b = ["%.02f" % l for l in ((xlocs * len(samples) / timebins) + (0.5 * binsize)) / samplerate]
    return xlocs, b, timebins


def plotMonoWav(mono_wav_path):
    # -*- coding: utf-8 -*-

    # 打开WAV文档
    f = wave.open(mono_wav_path, "rb")
    # 读取格式信息
    # (nchannels, sampwidth, framerate, nframes, comptype, compname)
    params = f.getparams()
    nchannels, sampwidth, framerate, nframes = params[:4]
    # print nchannels, sampwidth, framerate, nframes
    # 读取波形数据
    str_data = f.readframes(nframes)
    f.close()
    # 将波形数据转换为数组
    wave_data = np.fromstring(str_data, dtype=np.short)
    wave_data.shape = -1, nchannels
    wave_data = wave_data.T
    time = np.arange(0, nframes) * (1.0 / framerate)
    # 绘制波形
    # pl.subplot(211)
    pl.plot(time, wave_data[0])
    # pl.subplot(212)
    # pl.plot(time, wave_data[1], c="g")
    pl.xlabel("time (seconds)")
    pl.show()

    return 0


def plotDuralWav(dural_wav_path):
    # 打开WAV文档
    f = wave.open(dural_wav_path, "rb")
    # 读取格式信息
    # (nchannels, sampwidth, framerate, nframes, comptype, compname)
    params = f.getparams()
    nchannels, sampwidth, framerate, nframes = params[:4]
    # print nchannels, sampwidth, framerate, nframes
    # 读取波形数据
    str_data = f.readframes(nframes)
    f.close()
    # 将波形数据转换为数组
    wave_data = np.fromstring(str_data, dtype=np.short)
    wave_data.shape = -1, nchannels
    wave_data = wave_data.T
    time = np.arange(0, nframes) * (1.0 / framerate)
    # 绘制波形
    pl.subplot(211)
    pl.plot(time, wave_data[0])
    pl.subplot(212)
    pl.plot(time, wave_data[1], c="g")
    pl.xlabel("time (seconds)")
    pl.show()

    return 0



"""
SpectralFlux.py
Compute the spectral flux between consecutive spectra
This technique can be for onset detection

rectify - only return positive values
"""
def spectralFlux(spectra, rectify=False):
    """
    Compute the spectral flux between consecutive spectra
    """
    spectralFlux = []
    
    # Compute flux for zeroth spectrum
    flux = 0
    for bin in spectra[0]:
        flux = flux + abs(bin)
      
    spectralFlux.append(flux)
    
    # Compute flux for subsequent spectra
    for s in range(1, len(spectra)):
        prevSpectrum = spectra[s - 1]
        spectrum = spectra[s]
        
        flux = 0
        for bin in range(0, len(spectrum)):
            diff = abs(spectrum[bin]) - abs(prevSpectrum[bin])
            
            # If rectify is specified, only return positive values
            if rectify and diff < 0:
                diff = 0
            
            flux = flux + diff
            
        spectralFlux.append(flux)
        
    return spectralFlux


"""
Frame class
ndarray subclass for time-series data
Last updated: 31 January 2014
"""
import math
from math import *

import numpy
import numpy.fft
from numpy import *
from numpy.lib import stride_tricks


import matplotlib.pyplot as plt

import pymir
from pymir import Spectrum, Transforms, LinearPredictiveAnalysis
import pyaudio


class Frame(numpy.ndarray):

    def __new__(subtype, shape, dtype=float, buffer=None, offset=0,
                strides=None, order=None):
        # Create the ndarray instance of our type, given the usual
        # ndarray input arguments.  This will call the standard
        # ndarray constructor, but return an object of our type.
        # It also triggers a call to InfoArray.__array_finalize__
        obj = numpy.ndarray.__new__(subtype, shape, dtype, buffer, offset, strides,
                                    order)

        obj.sampleRate = 0
        obj.channels = 1
        obj.format = pyaudio.paFloat32

        # Finally, we must return the newly created object:
        return obj

    def __array_finalize__(self, obj):
        # ``self`` is a new object resulting from
        # ndarray.__new__(InfoArray, ...), therefore it only has
        # attributes that the ndarray.__new__ constructor gave it -
        # i.e. those of a standard ndarray.
        #
        # We could have got to the ndarray.__new__ call in 3 ways:
        # From an explicit constructor - e.g. InfoArray():
        #    obj is None
        #    (we're in the middle of the InfoArray.__new__
        #    constructor, and self.info will be set when we return to
        #    InfoArray.__new__)
        if obj is None:
            return
        # From view casting - e.g arr.view(InfoArray):
        #    obj is arr
        #    (type(obj) can be InfoArray)
        # From new-from-template - e.g infoarr[:3]
        #    type(obj) is InfoArray
        #
        # Note that it is here, rather than in the __new__ method,
        # that we set the default value for 'info', because this
        # method sees all creation of default objects - with the
        # InfoArray.__new__ constructor, but also with
        # arr.view(InfoArray).

        self.sampleRate = getattr(obj, 'sampleRate', None)
        self.channels = getattr(obj, 'channels', None)
        self.format = getattr(obj, 'format', None)

        # We do not need to return anything

    #####################
    # Frame methods
    #####################

    def cqt(self):
        """
        Compute the Constant Q Transform (CQT)
        """
        return Transforms.cqt(self)

    def dct(self):
        """
        Compute the Discrete Cosine Transform (DCT)
        """
        return Transforms.dct(self)

    def energy(self, windowSize=256):
        """
        Compute the energy of this frame
        """
        N = len(self)

        window = numpy.hamming(windowSize)
        window.shape = (windowSize, 1)

        n = N - windowSize  # number of windowed samples.

        # Create a view of signal who's shape is (n, windowSize). Use
        # stride_tricks such that each stide jumps only one item.
        p = numpy.power(self, 2)
        s = stride_tricks.as_strided(
            p, shape=(n, windowSize), strides=(self.itemsize, self.itemsize))
        e = numpy.dot(s, window) / windowSize
        e.shape = (e.shape[0], )
        return e

    def lpcc(self, lpcorder=None, cepsorder=None):
        '''
        Function: lpcc
        Summary: Computes the linear predictive cepstral compoents. Note: Returned values are in the frequency domain. LPCC is computed through LPC.
        Examples: audiofile = AudioFile.open('file.wav',16000)
              frames = audiofile.frames(512,np.hamming)
              for frame in frames:
                frame.lpcc()
        Attributes:
            @param (self):
            @param (lpcorder) default=None: The input order to compute the LPC coefficents.
            @param (cepsorder) default=None: The output order to compute the LPCC coefficents.
        Returns: A list of LPCC components with size order +1 or len(seq), depending on if cepsorder is None
        '''
        coefs, err_term = LinearPredictiveAnalysis.lpc(self, lpcorder)
        return LinearPredictiveAnalysis.lpcc(coefs, err_term, cepsorder)

    def lpc(self, order=None):
        '''
        Function: lpc
        Summary: Computes for each given sequence the LPC ( Linear predictive components ) sequence.
        Examples: audiofile = AudioFile.open('file.wav',16000)
                  frames = audiofile.frames(512,np.hamming)
                  for frame in frames:
                    frame.lpc()
        Attributes:
            @param (seq):A sequence of time-domain frames, usually obtained by .frames()
            @param (order) default=None: Size of the returning cepstral components. If None is given,
                                         we use len(seq) as default, otherwise order +1
        Returns: A list of lpc coefficents
        '''
        # Only return the coefficients not the error term (in [1])
        return LinearPredictiveAnalysis.lpc(self, order)[0]

    def lsp(self,order=None,rectify=True):
        '''
        Function: lsp
        Summary: Computes Line spectrum pairs ( also called  line spectral frequencies [lsf]). Does not use any fancy algorithm except np.roots to solve
    for the zeros of the given polynom A(z) = 0.5(P(z) + Q(z))
        Examples: audiofile = AudioFile.open('file.wav',16000)
                  frames = audiofile.frames(512,np.hamming)
                  for frame in frames:
                    frame.lsp()
        Attributes:
            @param (self):
            @param (order) default=None:Order of lpc coefficients. Return array has size order + 1. Default is the length of the current frame
            @param (rectify) default=True: Specifies if the return values are only positive. If rectify is False it also returns the (symmetric) negative values
        Returns: A list of size order/ len(frames) (if nothing is specifed), which represents the line spectrum pairs.
        '''
        coefs, _ = LinearPredictiveAnalysis.lpc(self, order)
        return LinearPredictiveAnalysis.lsp(coefs,rectify)

    def autocorr(self, order=None):
        '''
        Function: autocorr
        Summary: Calculates the autocorrelation with the given order
        Examples:   f = AudioFile.open('audiofile.wav',16000)
                    for frame in f.frames(512,numpy.hamming):
                        frame.autocorr()
        Attributes:
            @param (self):
            @param (order) default=None: The order ( order +1 is length of the returned array) of the auto correlation.
                                         If order is None we use len(frame)-1 as default
        Returns:Array of length order +1 with the autocorrelation coefficients
        '''
        if order is None:
            order = len(self) - 1
        return [sum(self[n] * self[n + tau] for n in xrange(len(self) - tau))
                for tau in xrange(order + 1)]

    def frames(self, frameSize, windowFunction=None):
        """
        Decompose this frame into smaller frames of size frameSize
        """
        frames = []
        start = 0
        end = frameSize
        while start < len(self):

            if windowFunction == None:
                frames.append(self[start:end])
            else:
                window = windowFunction(frameSize)
                window.shape = (frameSize, 1)
                window = numpy.squeeze(window)
                frame = self[start:end]
                if len(frame) < len(window):
                    # Zero pad
                    frameType = frame.__class__.__name__

                    sampleRate = frame.sampleRate
                    channels = frame.channels
                    format = frame.format

                    diff = len(window) - len(frame)
                    frame = numpy.append(frame, [0] * diff)

                    if frameType == "AudioFile":
                        frame = frame.view(pymir.AudioFile)
                    else:
                        frame = frame.view(Frame)

                    # Restore frame properties
                    frame.sampleRate = sampleRate
                    frame.channels = channels
                    frame.format = format

                windowedFrame = frame * window
                frames.append(windowedFrame)

            start = start + frameSize
            end = end + frameSize

        return frames

    def framesFromOnsets(self, onsets):
        """
        Decompose into frames based on onset start time-series
        """
        frames = []
        for i in range(0, len(onsets) - 1):
            frames.append(self[onsets[i]: onsets[i + 1]])

        return frames

    def play(self):
        """
        Play this frame through the default playback device using pyaudio (PortAudio)
        Note: This is a blocking operation.
        """
        # Create the stream
        p = pyaudio.PyAudio()
        stream = p.open(
            format=self.format, channels=self.channels, rate=self.sampleRate, output=True)

        # Write the audio data to the stream
        audioData = self.tostring()
        stream.write(audioData)

        # Close the stream
        stream.stop_stream()
        stream.close()
        p.terminate()

    def plot(self):
        """
        Plot the frame using matplotlib
        """
        plt.plot(self)
        plt.xlim(0, len(self))
        plt.ylim(-1.5, 1.5)
        plt.show()

    def rms(self):
        """
        Compute the root-mean-squared amplitude
        """
        sum = 0
        for i in range(0, len(self)):
            sum = sum + self[i] ** 2

        sum = sum / (1.0 * len(self))

        return math.sqrt(sum)

    # Spectrum
    def spectrum(self):
        """
        Compute the spectrum using an FFT
        Returns an instance of Spectrum
        """
        return Transforms.fft(self)

    def zcr(self):
        """
        Compute the Zero-crossing rate (ZCR)
        """
        zcr = 0
        for i in range(1, len(self)):
            if (self[i - 1] * self[i]) < 0:
                zcr = zcr + 1

        return zcr / (1.0 * len(self))



# Opening an audio file (AudioFile class)
import math

import numpy

from pymir import AudioFile


def calcLPCC(wav_path, frame_size=1024, lpcc_order=12):
    wavData = AudioFile.open(wav_path)
    lpcc_feature = []
    # Decomposing into frames
    # Fixed frame size
    windowFunction = numpy.hamming
    fixedFrames = wavData.frames(frame_size, windowFunction)
    for frame_item in fixedFrames:
        frame_lpcc_feature = frame_item.lpcc(lpcc_order)
        drop_flag = False
        for number in frame_lpcc_feature:
            if math.isinf(number) or math.isnan(number):
                drop_flag = True
                break
        if not drop_flag:
            lpcc_feature.append(frame_lpcc_feature)
    # print lpcc_feature
    # print len(lpcc_feature)
    lpcc_feature = numpy.array(lpcc_feature)

    return lpcc_feature

# calcLPCC('reading5-m-yxx.wav')



import librosa
import numpy as np


def calcSpectral_Centroid(wav_path):
    y, sr = librosa.load(wav_path)
    cent = librosa.feature.spectral_centroid(y=y, sr=sr)
    return cent


def calcSpectral_Bandwidth(wav_path):
    y, sr = librosa.load(wav_path)
    spec_bw = librosa.feature.spectral_bandwidth(y=y, sr=sr)

    return spec_bw


def calacSpectral_Contrast(wav_path):
    y, sr = librosa.load(wav_path)
    S = np.abs(librosa.stft(y))
    contrast = librosa.feature.spectral_contrast(S=S, sr=sr)

    # import matplotlib.pyplot as plt
    # plt.figure()
    # plt.subplot(2, 1, 1)
    # librosa.display.specshow(librosa.amplitude_to_db(S, ref=np.max), y_axis='log')
    # plt.colorbar(format='%+2.0f dB')
    # plt.title('Power spectrogram')
    # plt.subplot(2, 1, 2)
    # librosa.display.specshow(contrast, x_axis='time')
    # plt.colorbar()
    # plt.ylabel('Frequency bands')
    # plt.title('Spectral contrast')
    # plt.tight_layout()
    return contrast


def calcRMSE(wav_path):
    y, sr = librosa.load(wav_path)
    rmse = librosa.feature.rmse(y=y)
    return rmse


def calcSpectral_Rolloff(wav_path):
    y, sr = librosa.load(wav_path)
    rolloff = librosa.feature.spectral_rolloff(y=y, sr=sr)
    return rolloff


def calcPoly_Features(wav_path, order=0):
    y, sr = librosa.load(wav_path)
    S = np.abs(librosa.stft(y))
    p = librosa.feature.poly_features(S=S, order=order)
    return p


def calcZero_crossing_rate(wav_path):
    y, sr = librosa.load(wav_path)
    zero_crossing_rate = librosa.feature.zero_crossing_rate(y)
    return zero_crossing_rate



from distutils.core import setup

NAME = 'baseZhang'
_MAJOR = 1
_MINOR = 5
_MICRO = 0
VERSION = '%d.%d.%d' % (_MAJOR, _MINOR, _MICRO)
DESCRIPTION = "mir-feature-tools @ZHANG Xu-long"


def long_description():
    readme = open('README.md', 'r').read()
    changelog = open('CHANGELOG.md', 'r').read()
    return changelog + '\n\n' + readme


setup(
    packages=['baseZhang', 'pymir'],
    # data_files=[('./', ['CHANGELOG.md', 'README.md']),
    #     ('./docs/', [ 'docs/install.md','docs/pymir.md','docs/pythonTutorial.md']),
    #             ('./docs/img/', ['docs/img/pycharm_command_line_arguments.png','docs/img/pycharm_create_new_project.png','docs/img/pycharm_create_new_project_pure_python.png','docs/img/pycharm_hello_open.png','docs/img/pycharm_new_file_input.png','docs/img/pycharm_new_python_file.png','docs/img/pycharm_open.png','docs/img/pycharm_output.png','docs/img/pycharm_run.png','docs/img/terminal_screenshot.png']),
    #            ]
    # ,
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description(),
    author="ZHANG Xu-long",
    author_email="fudan0027zxl@gmail.com",
    license="BSD",
    url="http://zhangxulong.site",
    keywords='audio music sound',
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Topic :: Multimedia :: Sound/Audio :: Analysis",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",

    ],
    # install_requires=['stft==0.5.2','numpy==1.12.1', 'pandas==0.19.2', 'matplotlib==2.0.0', 'h5py==2.7.0', 'tqdm==4.11.2',
    #                   'PyAudio==0.2.11', 'pydub==0.18.0', 'pyPdf==1.13', 'PyYAML==3.12', 'six==1.10.0',
    #                   'SoundFile==0.9.0.post1', 'Theano==0.9.0', 'scikit-learn==0.18.1', 'Keras==1.2.2',
    #                   'librosa==0.5.0'],
)
